'''
File: DmpTime.py
Author: Duan Kaikai
Email: duankk@pmo.ac.cn
'''

from astropy.time import Time, TimeDelta


def UTC2MET(UTC):
    '''
    from the UTC time to DAMPE's MET
    '''
    T0 = Time('2013-01-01T00:00:00', scale='utc')
    T = Time(UTC, scale='utc')
    dT = T - T0
    MET = dT.sec
    return MET


def MET2UTC(MET):
    '''
    from DAMPE's MET to UTC time
    '''
    T0 = Time('2013-01-01T00:00:00', scale='utc')
    dT = TimeDelta(MET, format='sec')
    T = T0 + dT
    UTC = T.iso
    return UTC


def now():
    '''
    Get the Time of programe running
    '''
    now = Time.now()
    return now.iso


if __name__ == '__main__':
    import argparse
    parser = argparse.ArgumentParser('Time Transformation between UTC and MET of DAMPE')
    parser.add_argument('cmd', type=str, help='UTC2MET or MET2UTC')
    parser.add_argument('inputTime', type=str, help='UTC for UTC2MET or MET for MET2UTC')
    args = parser.parse_args()
    outputTime = eval(f'{args.cmd}({args.inputTime})')
    print('cmd:', args.cmd)
    print('inputTime:', args.inputTime)
    print('outputTime:', outputTime)
