import yaml
from astropy.coordinates import SkyCoord
import astropy.units as u
from dataclasses import dataclass

@dataclass
class Region:
    name: str
    xref: u.quantity.Quantity
    yref: u.quantity.Quantity
    rad: u.quantity.Quantity
    frame: str

    def __post_init__(self):
        self.skycrd = SkyCoord(self.xref, self.yref, frame=self.frame)


class FitRegion:
    def __init__(self, regionFile) -> None:
        self.__from_yaml(regionFile)

    def __from_yaml(self, regionFile):
        with open(regionFile, 'r') as f:
            fitRegion = yaml.load(f)

        self.inRegionList = []
        self.outRegionList = []
        try:
            inRegionDict = fitRegion['inRegion']
            for srcName, srcDict in inRegionDict.items():
                xref = float(srcDict['xref']['value']) * u.Unit(srcDict['xref']['unit'])
                yref = float(srcDict['yref']['value']) * u.Unit(srcDict['yref']['unit'])
                rad = float(srcDict['rad']['value']) * u.Unit(srcDict['rad']['unit'])
                frame = srcDict['frame']
                region = Region(srcName, xref, yref, rad, frame)
                self.inRegionList.append(region)
        except:
            pass

        try:
            outRegionDict = fitRegion['outRegion']
            for srcName, srcDict in outRegionDict.items():
                xref = float(srcDict['xref']['value']) * u.Unit(srcDict['xref']['unit'])
                yref = float(srcDict['yref']['value']) * u.Unit(srcDict['yref']['unit'])
                rad = float(srcDict['rad']['value']) * u.Unit(srcDict['rad']['unit'])
                frame = srcDict['frame']
                region = Region(srcName, xref, yref, rad, frame)
                self.outRegionList.append(region)
        except:
            pass


if __name__ == '__main__':
    xref = 3*u.degree
    print(xref)
    print(type(xref))
    region = Region('src', 3*u.degree, 4*u.degree, 5*u.degree, 'fk5')
    print(region)
    print(type(region))
    print(region.skycrd)
    fitRegion = FitRegion('fitRegion.yaml')
    print(fitRegion)
    print(type(fitRegion))
    print(fitRegion.inRegionList)
    print(fitRegion.outRegionList)