'''
File: Parameter.py
Author: Duan Kai-Kai
Email: duankk@pmo.ac.cn
'''

'''example
    >>> from Parameter import Parameter
    >>> par = {'free': '0',
                'max': '360',
                'min': '-360',
               'name': 'RA',
              'scale': '1',
              'value': '0.0377'}

    >>> par = {'error': '0.07322381815',
                'free': '1',
                 'max': '100000',
                 'min': '1e-05',
                'name': 'Prefactor',
               'scale': '1e-12',
               'value': '0.4869737351'}

    >>> ra = Parameter(par)
    >>> print ra.name, ra.scale, ra.value, ra.free, ra.min, ra.max

    >>> ra.GetValue
    >>> ra.GetRange
    >>> ra.SetValue(new_value)
    >>> ra.SetRange(new_min, new_max)
'''


class Parameter:
    def __init__(self, parDict):
        self.parDict = parDict

        self.name = self.parDict['name']
        self.free = bool(float(self.parDict['free']))
        self.scale = float(self.parDict['scale'])
        self.value = float(self.parDict['value'])
        self.min = float(self.parDict['min'])
        self.max = float(self.parDict['max'])
        if 'error' in list(self.parDict.keys()):
            self.error = float(self.parDict['error'])

    def __str__(self):
        return str(self.parDict)

    def __repr__(self):
        return str(self.parDict)

    @property
    def GetValue(self):
        return self.value * self.scale

    @property
    def GetError(self):
        return self.error * self.scale

    @property
    def GetRange(self):
        return (self.min * self.scale, self.max * self.scale)

    def SetFree(self, new_free):
        self.parDict['free'] = new_free
        self.free = new_free

    def SetScale(self, new_scale):
        self.parDict['scale'] = new_scale
        self.scale = new_scale

    def SetValue(self, new_value):
        self.parDict['value'] = new_value
        self.value = new_value

    def SetError(self, new_error):
        self.parDict['error'] = new_error
        self.error = new_error

    def SetRange(self, new_min, new_max):
        self.parDict['min'] = new_min
        self.parDict['max'] = new_max
        self.min = new_min
        self.max = new_max
