import os
import yaml
import numpy as np
from pprint import pprint
from DmpST import DmpTime, Events, LikelihoodAnalysis, Model
from plotSED import plotSED
from plotTSmap import plotTSmap
from plotSpectrum import plotSpectrum


normPars = ['Prefactor', 'Normalization', 'norm', 'value', 'Integral']
def BroadBandFit(kwargs):
    evfile = kwargs['evfile']
    scfile = kwargs['scfile']
    ltcube = kwargs['ltcube']
    path = kwargs['path']
    if not os.path.exists(path):
        os.mkdir(path)
    srcName = kwargs['srcName']
    srcName0 = f'{path}/{srcName}'

    GorC = kwargs['GorC']
    xref = kwargs['xref']
    yref = kwargs['yref']
    rad = kwargs['rad']
    freerad = kwargs['freerad']
    binsz = kwargs['binsz']
    proj = kwargs['proj']

    emin = kwargs['emin']
    emax = kwargs['emax']
    ebins = kwargs['ebins']

    UTCmin = kwargs['UTCmin']
    UTCmax = kwargs['UTCmax']
    METmin = DmpTime.UTC2MET(UTCmin)
    METmax = DmpTime.UTC2MET(UTCmax)

    cosThetamin = kwargs['cosThetamin']
    cosThetamax = kwargs['cosThetamax']
    evtype = kwargs['evtype']
    irf = kwargs['irf']

    events = srcName0 + '_events.fits'
    cmap = srcName0 + '_cmap.fits'
    ccube = srcName0 + '_ccube.fits'

    allevents = Events.Events(evfile)
    ROIevents = allevents.CircleSelect(GorC, xref, yref, rad, METmin, METmax, emin, emax, cosThetamin, cosThetamax, evtype, events)

    nxpix = int(2 * freerad / binsz)
    nypix = int(2 * freerad / binsz)
    ROIevents.Cmap(cmap, nxpix, nypix, binsz, GorC, xref, yref, proj)
    ROIevents.Ccube(ccube, nxpix, nypix, binsz, GorC, xref, yref, proj, emin, emax, ebins)

    #srcmdl = srcName0 + '_input.xml'
    srcmdl = srcName0 + '_input.yaml'
    srcmaps = srcName0 + '_srcmaps.fits'
    kwd = {'srcName': srcName0,
            'ltcube': ltcube,
             'ccube': ccube,
           'srcmaps': srcmaps,
            'srcmdl': srcmdl,
              'emin': emin,
              'emax': emax,
       'cosThetamin': cosThetamin,
       'cosThetamax': cosThetamax,
               'irf': irf,
            'evtype': evtype,
          'CubeType': 'Map'}

    Binned = LikelihoodAnalysis.BinnedAnalysis(**kwd)
    Binned.Fit(hesse=True, Npred=True, Flux=True, TsValue=True, srcmaps=True, mdlcube=True)

    outfmt = srcName0 + '_%s_BroadBand_binned.yaml'
    Binned.SaveFitModel(outfmt%'fitModel')
    Binned.SaveFitResults(outfmt%'results')
    Binned.SaveFitCovariance(outfmt%'covariance')
    plotSpectrum(path, srcName, outfmt%'fitModel')

    flux, error = Binned.Fluxes[srcName]
    TsValue = Binned.TsValues[srcName]
    #upperLimit = Binned.FluxUpperLimits[srcName]
    np.savetxt(srcName0 + '_flux_BroadBand_binned.dat', np.array([emin, emax, flux, error, TsValue]))

    ##resTSmap MUST before srcTSMap
    # Binned.resTSmap(srcName, srcName0 + '_resTSmap.fits')
    # plotTSmap(srcName0 + '_resTSmap.fits')

    # Binned.srcTSmap(srcName, srcName0 + '_srcTSmap.fits')
    # plotTSmap(srcName0 + '_srcTSmap.fits')

def BinbyBinFit(kwargs):
    path = kwargs['path']
    srcName = kwargs['srcName']
    srcName0 = f'{path}/{srcName}'
    Emin = kwargs['emin']
    Emax = kwargs['emax']
    Ebins = kwargs['ebins']
    ltcube = kwargs['ltcube']
    cosThetamin = kwargs['cosThetamin']
    cosThetamax = kwargs['cosThetamax']
    evtype = kwargs['evtype']
    irf = kwargs['irf']

    ccube = srcName0 + '_ccube.fits'
    srcmaps = srcName0 + '_srcmaps.fits'
    srcmdl = srcName0 + '_fitModel_BroadBand_binned.yaml'
    model = Model.Model(srcmdl)
    for srcname, Source in list(model.SrcDict.items()):
        for parName in list(Source.FreeParsDict.keys()):
            if parName not in normPars:
                model.modelDict[srcname]['spectrum'][parName]['free'] = '0'
    model.SaveModel(srcmdl)

    Ebounds = np.logspace(np.log10(Emin), np.log10(Emax), Ebins+1)
    Emins = Ebounds[:-1]
    Emaxs = Ebounds[1:]

    from multiprocessing import Pool
    args = [(srcName0, ltcube, ccube, srcmaps, srcmdl, emin, emax, cosThetamin, cosThetamax, irf, evtype) for emin, emax in zip(Emins, Emaxs)]
    pool = Pool(Ebins)
    results = pool.map(BinbyBinFit_mp, args)
    pool.close()

    np.savetxt(srcName0 + '_flux_SED_binned.dat', np.array(results))
    plotSED(path, srcName, srcName0 + '_flux_SED_binned.dat', Emin, Emax)

def BinbyBinFit_mp(args):
    srcName0 = args[0]
    _, srcName = srcName0.split('/')
    emin = args[5]
    emax = args[6]
    kwd = {'srcName': args[0],
            'ltcube': args[1],
             'ccube': args[2],
           'srcmaps': args[3],
            'srcmdl': args[4],
              'emin': args[5],
              'emax': args[6],
       'cosThetamin': args[7],
       'cosThetamax': args[8],
               'irf': args[9],
            'evtype': args[10],
          'CubeType': 'Map'}

    SED = LikelihoodAnalysis.BinnedAnalysis(**kwd)
    SED.Fit(Npred=True, Flux=True, TsValue=True, upperLimit=True)

    outfmt=srcName0+'_%s_{0:.2f}_{1:.2f}_binned.yaml'.format(emin, emax)
    SED.SaveFitModel(outfmt%'fitModel')
    SED.SaveFitResults(outfmt%'results')

    flux, error = SED.Fluxes[srcName]
    TsValue = SED.TsValues[srcName]
    upperLimit = SED.FluxUpperLimits[srcName]
    return emin, emax, flux, error, TsValue, upperLimit


if __name__ == '__main__':
    import argparse
    parser = argparse.ArgumentParser(description='Binned Likelihood Analysis and SED Script')
    parser.add_argument('evfile', type=str, help='name of evfile.')
    parser.add_argument('scfile', type=str, help='name of scfile.')
    parser.add_argument('ltcube', type=str, help='name of ltcube.')
    parser.add_argument('path', type=str, help='path of outfile.')
    parser.add_argument('srcName', type=str, help='name of target source')
    parser.add_argument('GorC', type=str, choices=['G', 'C'], help='G(GAL); C(CEL)')
    parser.add_argument('xref', type=float, help='L or RA of target source (deg)')
    parser.add_argument('yref', type=float, help='B or DEC of target source (deg)')
    parser.add_argument('rad', type=float, help='length of ROI rad')
    parser.add_argument('freerad', type=float, help='length of ROI free rad')
    parser.add_argument('binsz', type=float, help='size of each bin for counts map or ccube')
    parser.add_argument('proj', type=str, help='projection for counts map or ccube')
    parser.add_argument('emin', type=float, help='min energy of photon for analysis (MeV)')
    parser.add_argument('emax', type=float, help='max energy of photon for analysis (MeV)')
    parser.add_argument('ebins', type=int, help='number of energy bin for analysis')
    parser.add_argument('UTCmin', type=str, help='min time for analysis (UTC)')
    parser.add_argument('UTCmax', type=str, help='max time for analysis (UTC)')
    parser.add_argument('--cosThetamin', type=float, help='min cosTheta of photon for analysis', default=0.5)
    parser.add_argument('--cosThetamax', type=float, help='max cosTheta of photon for analysis', default=1.0)
    parser.add_argument('--evtype', type=int, help='evtype 0(LET), 1(HET) or 2()', default=2)
    parser.add_argument('--irf', type=str, help='path of irf', default='../DmpIRFs')
    args = parser.parse_args()
    kwargs = {'srcName': args.srcName,
                 'path': args.path,
               'evfile': args.evfile,
               'scfile': args.scfile,
               'ltcube': args.ltcube,
                 'GorC': args.GorC,
                 'xref': args.xref,
                 'yref': args.yref,
                  'rad': args.rad,
              'freerad': args.freerad,
                'binsz': args.binsz,
                 'proj': args.proj,
                 'emin': args.emin,
                 'emax': args.emax,
                'ebins': args.ebins,
               'UTCmin': args.UTCmin,
               'UTCmax': args.UTCmax,
          'cosThetamin': args.cosThetamin,
          'cosThetamax': args.cosThetamax,
               'evtype': args.evtype,
                  'irf': args.irf}
    BroadBandFit(kwargs)
    BinbyBinFit(kwargs)
