import os
import numpy as np
import matplotlib.pyplot as plt
from DmpST import Events, DmpTime
from plotInfo import plotEvents, plotTimeBin, plotEnergyBin, plotCmap


def GetInfo(args):
    evfile = args[0]
    if not os.path.exists(evfile):
        raise FileNotFoundError('The evfile not be found, please check it.')

    srcName = args[1]
    if not os.path.exists(srcName):
        os.mkdir(srcName)

    GorC, xref, yref, rad, binsz, proj = args[2]
    emin, emax, ebins = args[3]
    UTCmin, UTCmax, Tperiod = args[4]
    METmin = DmpTime.UTC2MET(UTCmin)
    METmax = DmpTime.UTC2MET(UTCmax)
    cosThetamin, cosThetamax, Trigger = args[5]

    events = Events.Events(evfile)
    #events = events.AllSkySelect(METmin, METmax, emin, emax, cosThetamin, cosThetamax, Trigger, srcName+'.fits')
    events = events.CircleSelect(GorC, xref, yref, rad, METmin, METmax, emin, emax, cosThetamin, cosThetamax, Trigger, f'{srcName}/events.fits')
    #events = events.BoxSelect(GorC, xmin, xmax, ymin, ymax, METmin, METmax, emin, emax, cosThetamin, cosThetamax, Trigger)

    nxpix = int(2. * rad / binsz)
    nypix = int(2. * rad / binsz)
    cmap = events.Cmap(f'{srcName}/cmap.fits', nxpix, nypix, binsz, GorC, xref, yref, proj)
    #hpxmap = events.HpxMap(srcName+'_HpxMap.fits', 6, 'RING', GorC, xref, yref)
    events.TimeBin(f'{srcName}/TimeBin.dat', METmin, METmax, Tperiod*24*3600)
    events.EnergyBin(f'{srcName}/EnergyBin.dat', emin, emax, ebins)

    plotCmap(cmap.data, cmap.wcs, f'{srcName}/cmap')
    #plotHpxMap(f'{srcName}/HpxMap.fits', f'{srcName}/HpxMap')
    plotTimeBin(f'{srcName}/TimeBin.dat', f'{srcName}/TimeBin')
    plotEnergyBin(f'{srcName}/EnergyBin.dat', f'{srcName}/EnergyBin')

    args = (events.Time, events.Energy, (emin, emax), (UTCmin, UTCmax, Tperiod), events.Theta, events.Phi, events.HETrigger)
    plotEvents(srcName, args, f'{srcName}/events')


if __name__ == "__main__":
    import argparse
    parser = argparse.ArgumentParser(description='Select DAMPE gamma-ray data and binned into Cmap')
    parser.add_argument('evfile', type=str, help='name of evfile')
    parser.add_argument('srcName', type=str, help='name of source of interest')
    parser.add_argument('GorC', type=str, choices=['G', 'C'], help='Galactic(G) or Celestial(C) coordinates system')
    parser.add_argument('xref', type=float, help='RA or L of source [deg]')
    parser.add_argument('yref', type=float, help='DEC or B of source [deg]')
    parser.add_argument('rad', type=float, help='rad of ROI [deg]', default=5)
    parser.add_argument('binsz', type=float, help='bin size of Cmap', default=0.1)
    parser.add_argument('proj', type=str, choices=['CAR', 'AIT'], help='projection of Cmap')
    parser.add_argument('emin', type=float, help='minimum energy [MeV]')
    parser.add_argument('emax', type=float, help='maximum energy [MeV]')
    parser.add_argument('ebins', type=int, help='number of energy bins')
    parser.add_argument('UTCmin', type=str, help='minumum UTC time')
    parser.add_argument('UTCmax', type=str, help='maximum UTC time')
    parser.add_argument('--Tperiod', type=float, help='period of time [day]', default=1)
    parser.add_argument('--cosThetamin', type=float, help='minumum cosTheta', default=0.5)
    parser.add_argument('--cosThetamax', type=float, help='maximum cosTheta', default=1)
    parser.add_argument('--evtype', type=int, help='Trigger type of data', default=2)
    args = parser.parse_args()

    args = (args.evfile, args.srcName, (args.GorC, args.xref, args.yref, args.rad, args.binsz, args.proj), (args.emin, args.emax, args.ebins), (args.UTCmin, args.UTCmax, args.Tperiod), (args.cosThetamin, args.cosThetamax, args.evtype))
    GetInfo(args)
