import sys
import numpy as np
import healpy as hp
import matplotlib as mpl
import matplotlib.pyplot as plt
from matplotlib import gridspec
from astropy.io import fits
from DmpST import DmpTime, Events
from astropy.visualization.wcsaxes.frame import EllipticalFrame


def plotTimeBin(infile, outfile):
    data = np.loadtxt(infile)
    tmins = data[:, 0]
    tmaxs = data[:, 1]
    times = (tmins + tmaxs) / 2
    counts = data[:, 2]
    
    '''
    fig, ax1 = plt.subplots()
    #ax1.vlines(days, 0, counts)
    #ax1.step(times, counts, 'c')
    ax1.set_yticks([0, 1, 2, 3, 4, 5])
    ax1.set_yticklabels([0, 1, 2, 3, 4, 5])
    ax1.set_ylabel('Counts')
    ax1.set_ylim(bottom=0)
    ax1.set_xlabel('days from 2016-01-01')

    ax2 = ax1.twinx()
    #ax2.axvline(MET, color='r')
    ax2.plot(day[Angle<60], Angle[Angle<60], '-')
    ax2.set_ylabel('Observed Angle [deg]')
    ax2.set_xlim(0, daymax)
    ax2.set_ylim(0, 60)
    '''
    plt.figure()
    plt.step(times, counts, 'k')
    plt.ylim(bottom=0)
    plt.xlabel('MET [s]')
    plt.ylabel('Counts')
    plt.grid()
    plt.savefig(f'{outfile}.png')
    plt.savefig(f'{outfile}.pdf')

def plotEnergyBin(infile, outfile):
    data = np.loadtxt(infile)
    emins = data[:, 0]
    emaxs = data[:, 1]
    energy = np.hstack([emins, emaxs[-1]])
    counts = data[:, 2]
    counts = np.hstack([counts[0], counts])

    plt.figure()
    plt.step(energy, counts, 'k')
    plt.xlim(emins[0], emaxs[-1])
    plt.ylim(bottom=0)
    plt.xscale('log')
    plt.xlabel('Energy [MeV]')
    plt.ylabel('Counts')
    plt.grid()
    plt.savefig(f'{outfile}.png')
    plt.savefig(f'{outfile}.pdf')

def plotPhaseBin(infile, outfile):
    data = np.loadtxt(infile)
    pmins = data[:, 0]
    pmaxs = data[:, 1]
    phase = np.hstack([pmins, pmaxs[-1]])
    counts = data[:, 2]
    counts = np.hstack([counts[0], counts])

    plt.figure()
    plt.step(phase, counts, 'k')
    plt.xlim(0, 1)
    plt.ylim(bottom=0)
    plt.xlabel('Phase')
    plt.ylabel('Counts')
    plt.grid()
    plt.savefig('{}.png'.format(outfile))
    plt.savefig('{}.pdf'.format(outfile))

def plotTimeBin2(infile1, infile2, outfile):
    data1 = np.loadtxt(infile1)
    data2 = np.loadtxt(infile2)
    tmins = data1[:, 0]
    tmaxs = data1[:, 1]
    times = (tmins + tmaxs) / 2
    counts1 = data1[:, 2]
    counts2 = data2[:, 2]

    plt.figure()
    plt.vlines(times, 0, counts1, 'r', label='LET')
    plt.vlines(times, counts1, counts2, 'b', label='HET')
    plt.xlim(tmins[0], tmaxs[-1])
    plt.ylim(bottom=0)
    plt.xlabel('MET time')
    plt.ylabel('Counts')
    plt.grid()
    plt.legend()
    plt.savefig('{}.png'.format(outfile))
    plt.savefig('{}.pdf'.format(outfile))

def plotEnergyBin2(infile1, infile2, outfile):
    data1 = np.loadtxt(infile1)
    data2 = np.loadtxt(infile2)
    emins = data1[:, 0]
    emaxs = data1[:, 1]
    energy = np.hstack([emins, emaxs[-1]])
    counts1 = data1[:, 2]
    counts2 = data2[:, 2]
    counts1 = np.hstack([counts1[0], counts1])
    counts2 = np.hstack([counts2[0], counts2])

    plt.figure()
    plt.step(energy, counts1, 'r', label='LET')
    plt.step(energy, counts1 + counts2, 'b', label='HET')
    plt.xlim(emins[0], emaxs[-1])
    plt.ylim(bottom=0)
    plt.xscale('log')
    plt.xlabel('Energy [MeV]')
    plt.ylabel('Counts')
    plt.grid()
    plt.legend()
    plt.savefig('{}.png'.format(outfile))
    plt.savefig('{}.pdf'.format(outfile))

def plotPhaseBin2(infile1, infile2, outfile):
    data1 = np.loadtxt(infile1)
    data2 = np.loadtxt(infile2)
    pmins = data1[:, 0]
    pmaxs = data1[:, 1]
    phase = np.hstack([pmins, pmaxs[-1]])
    counts1 = data1[:, 2]
    counts2 = data2[:, 2]
    counts1 = np.hstack([counts1[0], counts1])
    counts2 = np.hstack([counts2[0], counts2])

    plt.figure()
    plt.step(phase, counts1, 'r', label='LET')
    plt.step(phase, counts1 + counts2, 'b', label='HET')
    plt.xlim(0, 1)
    plt.ylim(bottom=0)
    plt.xlabel('Phase')
    plt.ylabel('Counts')
    plt.grid()
    plt.legend()
    plt.savefig('{}.png'.format(outfile))
    plt.savefig('{}.pdf'.format(outfile))

def plotEvents(srcName, times, outfile):
    Time, Energy = times[0], times[1]
    Emin, Emax = times[2]
    UTCmin, UTCmax, Tperiod = times[3]
    METmin = DmpTime.UTC2MET(UTCmin)
    METmax = DmpTime.UTC2MET(UTCmax)
    tbins = int((METmax - METmin) / (Tperiod * 24 * 3600))
    
    Theta, Phi, HETrigger = times[4], times[5], times[6]
    cosTheta = np.cos(np.deg2rad(Theta))

    fig = plt.figure()
    plt.plot(Time[HETrigger == 0], Energy[HETrigger == 0], '.r', label='LET')
    plt.plot(Time[HETrigger == 1], Energy[HETrigger == 1], '.b', label='HET')
    plt.xlabel("Time")
    plt.ylabel("Energy [MeV]")
    plt.yscale('log')
    plt.xlim(METmin, METmax)
    plt.ylim(Emin,Emax)
    plt.legend()

    METbounds = np.loadtxt(f'{srcName}/METbounds.dat')
    try:
        METbounds_left = METbounds[:, 0]
        METbounds_right = METbounds[:, 1]
    except:
        METbounds_left = [METbounds[0]]
        METbounds_right = [METbounds[1]]
    plt.xticks(list(METbounds_left) + list(METbounds_right), [DmpTime.MET2UTC(METbound).split()[0] for METbound in list(METbounds_left) + list(METbounds_right)], rotation=-90)

    ax1 = plt.gca()
    ax1.set_position([0.125, 0.25, 0.8, 0.4])
    ax2 = fig.add_axes([0.125, 0.65, 0.8, 0.3])
    ax2.set_xticks([])

    Observed = np.loadtxt(f'{srcName}/ObservedTime.dat')
    time = Observed[:, 0]
    Angle = Observed[:, 2]

    ax2.plot(time[Angle<60], Angle[Angle<60], '-')
    ax2.set_ylabel('Observed Angle [deg]')
    ax2.set_xlim(METmin, METmax)
    ax2.set_ylim(0, 60)

    for i in range(len(METbounds_left) - 1):
        if abs(METbounds_right[i] - METmax) < 1e4:
            break
        ax1.axvspan(METbounds_right[i], METbounds_left[i+1], color='gray', alpha=0.5)
        ax2.axvspan(METbounds_right[i], METbounds_left[i+1], color='gray', alpha=0.5)

    #SolarFlare
    #METmin = DmpTime.UTC2MET('2017-09-08')
    #METmax = DmpTime.UTC2MET('2017-09-13')
    #ax1.axvspan(METmin, METmax, color='gray', alpha=0.5)
    #ax2.axvspan(METmin, METmax, color='gray', alpha=0.5)

    '''
    plt.subplot(122)
    plt.plot(Phi[HETrigger == 0], cosTheta[HETrigger == 0], '.r', label='LET')
    plt.plot(Phi[HETrigger == 1], cosTheta[HETrigger == 1], '.b', label='HET')
    plt.gca().yaxis.set_label_position('right')
    plt.gca().yaxis.set_ticks_position('right')
    plt.xlabel(r'$\phi$')
    plt.ylabel(r'cos$\theta$')
    plt.xlim(0, 360)
    plt.ylim(0.5, 1)
    #plt.legend()
    plt.grid()
    '''
    plt.savefig('{}.png'.format(outfile))
    plt.savefig('{}.pdf'.format(outfile))

def plotCmap(data, wcs, outfile):
    cmap = mpl.cm.gnuplot2
    #norm = mpl.colors.NoNorm(vmin=1, vmax=np.amax(data))
    #norm = mpl.colors.LogNorm(vmin=0.1, vmax=np.amax(data))
    norm = mpl.colors.Normalize(vmin=np.amin(data), vmax=np.amax(data))
    #norm = mpl.colors.PowerNorm(0.5, vmin=np.amin(data), vmax=np.amax(data))

    fig = plt.figure()
    #gs = gridspec.GridSpec(2, 2)
    #ax = fig.add_subplot(gs[:, 0], projection=wcs)
    #ax = plt.subplot(projection=wcs, frame_class=EllipticalFrame)
    ax = plt.subplot(projection=wcs)
    img = ax.imshow(data.T, cmap=cmap, norm=norm, origin='lower')
    fig.colorbar(img, ax=ax, cmap=cmap, norm=norm)
    ax.grid(color='white', ls='--', alpha=0.5)
    ax.set_xlabel(wcs.axis_type_names[0])
    ax.set_ylabel(wcs.axis_type_names[1])

    '''
    nx, ny = data.shape
    xdata = data[int(ny/2), :]
    ydata = data[:, int(nx/2)]
    ax1 = fig.add_subplot(gs[0, 1])
    ax1.plot(xdata)
    ax1.set_xlabel('horizontal')
    ax1.set_ylabel('Number of photons')
    ax2 = fig.add_subplot(gs[1, 1])
    ax2.plot(ydata)
    ax2.set_xlabel('vertical')
    ax2.set_ylabel('Number of photons')
    '''
    plt.savefig(f'{outfile}.png')
    plt.savefig(f'{outfile}.pdf')

def plotHpxMap(infile, outfile):
    m = hp.read_map(infile)
    hp.mollview(m)
    plt.savefig('{}.png'.format(outfile))
    plt.savefig('{}.pdf'.format(outfile))


if __name__ == '__main__':
    filename = sys.argv[1]
    hdulist = fits.open(filename)
    data = hdulist[1].data
    Time = data.field('Time')
    Energy = data.field('Energy')
    Theta = data.field('Theta')
    Phi = data.field('Phi')
    HETrigger = data.field('HETrigger')
    
    #Emin, Emax, ebins = float(sys.argv[2]), float(sys.argv[3]), int(sys.argv[4])
    #UTCmin, UTCmax, Tperiod = sys.argv[5], sys.argv[6], float(sys.argv[7])
    #times = (Time, Energy, (Emin, Emax), (UTCmin, UTCmax, Tperiod), Theta, Phi, HETrigger)
    #plotEvents(times, 'Events')

    events = Events.Events(filename)
    #METmin = DmpTime.UTC2MET(UTCmin)
    #METmax = DmpTime.UTC2MET(UTCmax)
    #events.TimeBin('TimeBin.dat', METmin, METmax, Tperiod * 24 * 3600)
    #events.EnergyBin('EnergyBin.dat', Emin, Emax, ebins)
    #plotTimeBin('TimeBin.dat', 'TimeBin')
    #plotEnergyBin('EnergyBin.dat', 'EnergyBin')

    events.HpxMap('HpxMap.fits', 6, 'RING', 'G', (0, 0))
    plotHpxMap('HpxMap.fits', 'HpxMap')
