import numpy as np
import matplotlib.pyplot as plt


def plotSED(path, srcName, fluxfile, Emin, Emax):
    data = np.loadtxt(fluxfile)
    emin = data[:, 0]
    emax = data[:, 1]
    flux = data[:, 2]
    error = data[:, 3]
    TSvalue = data[:, 4]
    upperLimit = data[:, 5]

    x = np.sqrt(emin * emax)
    xerr = np.array([x-emin, emax-x])
    y = x**2 * flux / (emax - emin)
    yerr = x**2 * error / (emax - emin)
    y_ul = x**2 * upperLimit / (emax - emin)
    y_ulerr = np.array([0.5*y_ul, [0]*len(y)])

    e, dnde = np.loadtxt(f'{path}/{srcName}_BroadBand_Spectrum.dat').T

    fig, ax = plt.subplots()
    condition = (Emin <= e) * (e <= Emax)
    plt.plot(e[condition], e[condition]**2 * dnde[condition])
    ax.errorbar(x[TSvalue>9], y[TSvalue>9], xerr=xerr[:,TSvalue>9], yerr=yerr[TSvalue>9], fmt='.k', label='DAMPE')
    ax.errorbar(x[TSvalue<9], y_ul[TSvalue<9], xerr=xerr[:,TSvalue<9], yerr=y_ulerr[:,TSvalue<9], fmt='.k', uplims=True)
    ax.set_xlim(Emin, Emax)
    ax.set_xscale('log')
    ax.set_yscale('log')
    ax.set_xlabel(r'Energy [MeV]')
    ax.set_ylabel(r'Energy Flux [MeV cm$^{-2}$ s$^{-1}$]')

    ax2 = ax.twinx()
    ax2.bar(x, TSvalue, width=(emax-emin), color='gray', edgecolor='gray', alpha=0.5)
    ax2.set_ylim(bottom=0)
    ax2.set_xscale('log')
    ax2.set_ylabel('TS value')

    plt.savefig(fluxfile.replace('.dat', '.png'))
    plt.savefig(fluxfile.replace('.dat', '.pdf'))

def plotSED2(fluxfile, Emin, Emax, flux2):
    data = np.loadtxt(fluxfile)
    emin = data[:, 0]
    emax = data[:, 1]
    flux = data[:, 2]
    error = data[:, 3]
    #TSvalue = data[:, 4]
    #upperLimit = data[:, 5]
        
    x = np.sqrt(emin * emax)
    xerr = np.array([x-emin, emax - x])
    y = x**2 * flux / (emax - emin)
    yerr = x**2 * error / (emax - emin)
    #y_ul = x**2 * upperLimit / (emax - emin)
    #y_ulerr = np.array([0.5*y_ul, [0]*len(y)])
    
    fig, ax = plt.subplots()
    ax.errorbar(x, y, xerr=xerr, yerr=yerr, fmt='.r', label='DAMPE')
    #ax.errorbar(x, y_ul, xerr=xerr, yerr=y_ulerr, fmt='.k', uplims=True)
    
    data = np.loadtxt(flux2)
    emin = data[:, 0]
    emax = data[:, 1]
    flux = data[:, 2]
    error = data[:, 3]
    x = np.sqrt(emin * emax)
    xerr = np.array([x-emin, emax - x])
    y = x**2 * flux / (emax - emin)
    yerr = x**2 * error / (emax - emin)
    ax.errorbar(x, y, xerr=xerr, yerr=yerr, fmt='.g', label='FERMI')

    plt.legend()
    ax.set_xlim(Emin, Emax)
    #ax.set_ylim(1e-4, 1e-1)
    ax.set_xscale('log')
    ax.set_yscale('log')
    ax.set_xlabel(r'Energy [MeV]')
    ax.set_ylabel(r'Energy Flux [$MeV cm^{-2} s^{-1}$]')
    plt.savefig(fluxfile.replace('.dat', '.png'))
    plt.savefig(fluxfile.replace('.dat', '.pdf'))

def plotSEDs(fluxList, Emin, Emax):
    for fluxfile in fluxList:
        data = np.loadtxt(fluxfile)
        emin = data[:, 0]
        emax = data[:, 1]
        flux = data[:, 2]
        error = data[:, 3]
        TSvalue = data[:, 4]
        upperLimit = data[:, 5]

        x = np.sqrt(emin * emax)
        xerr = np.array([x-emin, emax - x])
        y = x**2 * flux / (emax - emin)
        yerr = x**2 * error / (emax - emin)
        y_ul = x**2 * upperLimit / (emax - emin)
        y_ulerr = np.array([0.5*y_ul, [0]*len(y)])

        plt.errorbar(x, y, xerr=xerr, yerr=yerr, fmt='.r', label='DAMPE')
        #plt.errorbar(x, y_ul, xerr=xerr, yerr=y_ulerr, fmt='.r', uplims=True)
    plt.legend()
    plt.xlim(Emin, Emax)
    plt.ylim(1e6, 1e12)
    plt.xscale('log')
    plt.yscale('log')
    plt.xlabel(r'Energy [MeV]')
    plt.ylabel(r'Energy Flux [$MeV cm^{-2} s^{-1}$]')
    plt.savefig(fluxfile.replace('.dat', '.png'))
    plt.savefig(fluxfile.replace('.dat', '.pdf'))


if __name__ == '__main__':
    import sys
    path = sys.argv[1]
    srcName = sys.argv[2]
    flux = sys.argv[3]
    Emin = float(sys.argv[4])
    Emax = float(sys.argv[5])
    plotSED(path, srcName, flux, Emin, Emax)
    #flux2 = sys.argv[6]
    #plotSED2(flux, Emin, Emax, flux2)
