from scipy.stats.stats import mode
import yaml
import numpy as np
import matplotlib.pyplot as plt
from DmpST import Spectrum, Model


def plotSpectrum(path, srcName, modelFile):
    with open(modelFile, 'r') as f_model:
        modelDict = yaml.load(f_model)
    srcDict = modelDict[srcName]
    SpecType = srcDict['spectrum']['type']
    SpecDict = srcDict['spectrum']
    spectrum = eval(f'Spectrum.{SpecType}(SpecDict)')
    e = np.logspace(3, 6, 100)
    dnde = spectrum.GetSpecFunc(e)
    np.savetxt(f'{path}/{srcName}_BroadBand_Spectrum.dat', np.array([e, dnde]).T)

    plt.figure()
    plt.plot(e, e**2 * dnde)
    plt.xscale('log')
    plt.yscale('log')
    plt.xlim(1e3, 1e6)
    plt.xlabel(r'Energy [MeV]')
    plt.ylabel(r'E$^2$ dN/dE [MeV cm$^{-2}$ s$^{-1}$]')
    plt.savefig(f'{path}/{srcName}_BroadBand_Spectrum.png')
    plt.savefig(f'{path}/{srcName}_BroadBand_Spectrum.pdf')


if __name__ == '__main__':
    import sys
    path = sys.argv[1]
    srcName = sys.argv[2]
    modelFile = sys.argv[3]
    plotSpectrum(path, srcName, modelFile)