import sys
import numpy as np
import matplotlib.pyplot as plt
from astropy.io import fits
from astropy.wcs import WCS
from DmpST import SkyMap


def load_ds9_cmap():
    # http://tdc-www.harvard.edu/software/saoimage/saoimage.color.html
    ds9_b = {
        'red': [[0.0, 0.0, 0.0],                         
                [0.25, 0.0, 0.0],
                [0.50, 1.0, 1.0],
                [0.75, 1.0, 1.0],
                [1.0, 1.0, 1.0]],
      'green': [[0.0, 0.0, 0.0],                                       
                [0.25, 0.0, 0.0],
                [0.50, 0.0, 0.0],
                [0.75, 1.0, 1.0],
                [1.0, 1.0, 1.0]],
       'blue': [[0.0, 0.0, 0.0],
                [0.25, 1.0, 1.0],
                [0.50, 0.0, 0.0],
                [0.75, 0.0, 0.0],
                [1.0, 1.0, 1.0]]                             
            }

    plt.register_cmap(name='ds9_b', data=ds9_b)
    plt.cm.ds9_b = plt.cm.get_cmap('ds9_b')
    return plt.cm.ds9_b

def plotTSmap(infile):
    tsmap = SkyMap.Cmap(infile)
    
    load_ds9_cmap()
    fig = plt.figure()
    fig.add_subplot(111, projection=tsmap.wcs)
    plt.imshow(tsmap.data.T, origin='lower', cmap='ds9_b')
    plt.colorbar()
    plt.grid(color='white', ls='--', alpha=0.3)
    plt.xlabel('RA')
    plt.ylabel('DEC')
    plt.savefig(infile.replace('fits', 'png'))
    plt.savefig(infile.replace('fits', 'pdf'))


if __name__ == '__main__':
    infile = sys.argv[1]
    plotTSmap(infile)